/*
 * dfc - show free disk space
 *
 * This is just a quick test to see how this program could work in C,
 * all credit goes to MilesAhead on DonationCoder.com forum;
 * http://www.donationcoder.com/forum/index.php?topic=33071.0
 *
 * Copyright 2012 Joergen Ibsen
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <windows.h>
#include <stdio.h>
#include <limits.h>

/* For 64-bit values we only need the suffixes up to EiB which is 60 bits */
const char *suffixes[] = { "KiB", "MiB", "GiB", "TiB", "PiB", "EiB" };

void print_iec_size(unsigned long long d, const char *dec)
{
	unsigned long long base = 1024;
	int suffixnum = 0;

	/* Find nearest power of 1024 below d, and corresponding suffix */
	while (base < ULLONG_MAX / 1024 && d >= base*1024)
	{
		base *= 1024;
		++suffixnum;
	}

	/* Convert to double and divide by base to get a floating point value
	   in the range [0:1024[ */
	printf("%.3f %s", (double)d / base, suffixes[suffixnum]);
}

void print_integer_recursive(unsigned long long d, const char *sep, int last)
{
	/* Check if this is the leading group of digits */
	if (d >= 1000)
	{
		/* Recursively print the groups of digits before this one */
		print_integer_recursive(d / 1000, sep, 0);

		/* Print this group of digits, zero padded since it was not the
		   first, and with a thousands separator after it, if not the last */
		printf("%03llu%s", d % 1000, last ? "" : sep);
	} else {
		/* Print this leading group of digits, with a thousands separator
		   after it, if not the last */
		printf("%llu%s", d, last ? "" : sep);
	}
}

void print_integer(unsigned long long d, const char *sep)
{
	print_integer_recursive(d, sep, 1);
}

void print_space(const char *path)
{
	ULARGE_INTEGER totalSize, freeSize, availSize;

	/* Show path being processed */
	printf("\n[%s]\n", path);

	/* Get free space info */
	if (!GetDiskFreeSpaceEx(path, &availSize, &totalSize, &freeSize))
	{
		printf("GetDiskFreeSpaceEx() failed with error code 0x%08X\n", GetLastError());
		exit(-1);
	}

	/* Show total size */
	printf("Total size\t: ");
	print_iec_size(totalSize.QuadPart, ".");
	printf(" (");
	print_integer(totalSize.QuadPart, ",");
	printf(" bytes)\n");

	/* Show free size */
	printf("Free size\t: ");
	print_iec_size(freeSize.QuadPart, ".");
	printf(" (");
	print_integer(freeSize.QuadPart, ",");
	printf(" bytes)\n");

	/* Show size available to user, if different */
	if (availSize.QuadPart != freeSize.QuadPart)
	{
		printf("Available size\t: ");
		print_iec_size(availSize.QuadPart, ".");
		printf(" (");
		print_integer(availSize.QuadPart, ",");
		printf(" bytes)\n");
	}
}

int main(int argc, char *argv[])
{
	char drive[] = "C:\0";
	int i;

	/* If no command line arguments, use current directory */
	if (argc < 2)
	{
		char currentpath[MAX_PATH];
		DWORD dwSize;

		/* Get current directory */
		dwSize = GetCurrentDirectory(MAX_PATH, currentpath);

		if (dwSize == 0 || dwSize >= MAX_PATH)
		{
			printf("GetCurrentDirectory() failed with error code 0x%08X\n", GetLastError());
			exit(-1);
		}

		/* Print space information for path */
		print_space(currentpath);

		return 0;
	}

	/* Loop over command line arguments */
	for (i = 1; i < argc; ++i)
	{
		const char *path = argv[i];

		/* If argument was a single character, we copy it to drive and use
		   that string to append a colon */
		if (strlen(path) == 1)
		{
			drive[0] = path[0];
			path = drive;
		}

		/* Print space information for path */
		print_space(path);
	}

	return 0;
}
